/** 
 *******************************************************************************
 * @file    uart_io.c
 * @brief   Uart Input/Output
 * @version V1.0.2.0
 * $Date:: 2019-07-19 15:39:36 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include	<stdlib.h>
#include	"uart_io.h"
/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup UART UART Sample Appli
 *  @{
 */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup UART_Private_variables UART Private Variables
 *  @{
 */
UINT8_t error       = (UINT8_t)0;			/*!< error flag.        */
UINT8_t cmp_flag    = (UINT8_t)0;			/*!< Receive complete flag.        */
UINT8_t rx_data[UART_DATA_LENGTH];
UINT8_t tx_data[UART_DATA_LENGTH];
UINT8_t write        = (UINT8_t)0;
UINT8_t rx_length    = (UINT8_t)0;
UINT8_t tx_length    = (UINT8_t)0;
/**
 *  @}
 */ /* End of group UART_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup UART_Private_fuctions UART Private Fuctions
 *  @{
 */
void UART_Config(void);
void uart_initialize(void);
void TX_Start(void);
void uart_data_set(UINT8_t length);
void Send_Display_Data(UINT8_t size, UINT8_t *pnt);
/*--------------------------------------------------*/
/** 
  * @brief  UART_Configure
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void UART_Config(void)
{
	/* (1) Select TXD2:P93,RXD2:P94 */
	UATCNG = UART_PORT_TRXD_PB;
	/* (2) Enable UART2  */
	POFFCR1 = UART_UART2_ENABLE;
	/* (3) Set TXD2,RXD2 */
	P93 = SET;
	P9FC3 = PORT_FUNC_ON;
	P9CR3 = PORT_OUTPUT;
	P9CR4 = PORT_INPUT;
	/* (4) Enable interrupt of Rx */
	__DI();
	EIR_INTRXD2 = 1;
	EIR_INTTXD2 = 1;
	__EI();
	/* (5) Set Tx 2 Stop Bit,No Parity,TXD=UART,Trans Clock=fcgck  */
	UART2CR1 = UART_TRANS_STOP_BIT_LENGTH | UART_PARITY | UART_TXD_UART_OUTPUT | UART_CLOCK;
	/* (6) Set RT Clock=17/17,No Noize Removal,Rx 1 Stop Bit */
	UART2CR2 = UART_RT_CLOCK | UART_NOIZE_CANCEL_TIME | UART_RECIEVE_STOP_BIT_LENGTH;
	/* (7) Set Baud Rate = 115200bps */
	UART2DR = UART_BAUD_RATE;
}

/**
 *  @}
 */ /* End of group UART_Private_functions */
/**
 *  @}
 */ /* End of group UART_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup UART_Exported_functions UART Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  UART Initialize.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void uart_initialize(void)
{
	/*------------------------*/
	/* UART Setting           */
	/*------------------------*/
	UART_Config();
}
/*--------------------------------------------------*/
/** 
  * @brief  UART TX Start
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void TX_Start(void)
{
	UART2CR1_TXE = UART_TXE_ENABLE;
}
/*--------------------------------------------------*/
/** 
  * @brief  UART Data setting.
  * @param  length:send data length
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void uart_data_set(UINT8_t length)
{
	write = 0;
	tx_length = length;
}
/*--------------------------------------------------*/
/** 
  * @brief  send display data
  * @param  size:send data size
  * @param  pnt:send buff pointer
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void Send_Display_Data(UINT8_t size, UINT8_t *pnt)
{
	UINT16_t	i;
	UINT8_t 	t_state = (UINT8_t)0;

	uart_data_set(size);
	for(i=0 ; i<=size ; i++) {
		tx_data[i] = pnt[i];
	}
	for(i=0;i<50;i++){
		__asm("nop");
	}
	t_state = UART2SR;
	while((t_state & 0x02) != 0) {
		t_state = UART2SR;
	}
	UART2CR1_TXE = UART_TXE_ENABLE;
	TD2BUF = tx_data[write];
	for(;;) {
		if(write == 0xFF) {
			break;
		}
		else {
			/* Do nothing */
		}
	}
}
/*--------------------------------------------------*/
/** 
  * @brief  Interrupt RXD2
  * @param  -
  * @return -
  * @retval -
  * @note   UART RXD2 interrupt
  */
/*--------------------------------------------------*/
void __interrupt IntRXD2(void)
{
	UINT8_t r_state = (UINT8_t)0;
	CHAR_t data = (CHAR_t)0;
	
	r_state = UART2SR;
	data = (CHAR_t)RD2BUF;
	
	/* RBFL0 & error check */
	if((r_state & 0xE0) == 0) {
		if ((data != -1) && (rx_length < UART_DATA_LENGTH)) {
			rx_data[rx_length] = (char)data;
			switch ((char)data) {
			/*--- CR: carriage return ---*/
			case '\r':
				/* no process */
				break;
			/*--- LF: line feed ---*/
			case '\n':
				cmp_flag = (UINT8_t)1;
				tx_length = rx_length;
				break;
			/*--- default ---*/
			default:
				rx_length++;
				/* no process */
				break;
			}
		}
		else {
			cmp_flag = (UINT8_t)1;
			error = ERROR_UART_RECEIVE;			/* Report an error */
		}
	}
	else {
		/* error or busy or buff empty */
		cmp_flag = 1;
		error = ERROR_UART_RECEIVE;				/* Report an error */
	}
}
/*--------------------------------------------------*/
/** 
  * @brief  Interrupt TXD2
  * @param  -
  * @return -
  * @retval -
  * @note   UART TXD2 interrupt
  */
/*--------------------------------------------------*/
void __interrupt IntTXD2(void)
{
	write++;
	
	if(write >= tx_length ){
		/* Tx Stop */
		tx_length = (UINT8_t)0;
		write = (UINT8_t)0xFF;
		UART2CR1_TXE = UART_TXE_DISABLE;
	}
	else {
		TD2BUF = tx_data[write];
	}
}
/**
 *  @}
 */ /* End of group UART_Exported_functions */

/**
 *  @}
 */ /* End of group UART */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
